@@ -3,7 +3,7 @@ import sys
 import os
 import re
 import pickle
-from multiprocessing import Process, Queue
+import multiprocessing
 import ConfigParser as configparser
 
 from PyQt4.QtCore import *
@@ -12,13 +12,13 @@ from PyQt4.QtGui import *
 
 from qtcustom import *
 import pytrailer as amt
+from downloader import TrailerDownloader, DownloadStatus
 
-
-categories = {'Just added':'/trailers/home/feeds/just_added.json',
-              'Exclusive':'/trailers/home/feeds/exclusive.json',
-              'Only HD':'/trailers/home/feeds/just_hd.json',
-              'Most popular':'/trailers/home/feeds/most_pop.json',
-              'Search':'/trailers/home/scripts/quickfind.php?callback=searchCallback&q='}
+categories = [('Just added', '/trailers/home/feeds/just_added.json'),
+              ('Exclusive', '/trailers/home/feeds/exclusive.json'),
+              ('Only HD', '/trailers/home/feeds/just_hd.json'),
+              ('Most popular', '/trailers/home/feeds/most_pop.json'),
+              ('Search', '/trailers/home/scripts/quickfind.php?callback=searchCallback&q=')]
 
 
 class PyTrailerWidget(QMainWindow):
@@ -31,17 +31,23 @@ class PyTrailerWidget(QMainWindow):
                                        'filters':pickle.dumps([])})
         self.config.read(self.configPath)
         self.movieDict = {}
-        self.readAheadTaskQueue = Queue()
-        self.readAheadDoneQueue = Queue()
+        self.readAheadTaskQueue = multiprocessing.Queue()
+        self.readAheadDoneQueue = multiprocessing.Queue()
+        self.trailerDownloadQueue = multiprocessing.Queue()
+        self.trailerDownloadDict = multiprocessing.Manager().dict()
 
         self.readAheadProcess = []
         for i in range(READ_AHEAD_PROC):
-            p = Process(target=movieReadAhead,
+            p = multiprocessing.Process(target=movieReadAhead,
                         args=(self.readAheadTaskQueue,
                               self.readAheadDoneQueue))
             p.start()
             self.readAheadProcess.append(p)
 
+        self.downloader = TrailerDownloader(self.trailerDownloadQueue,
+                               self.trailerDownloadDict,
+                               2)
+        self.downloader.start()
         self.init_widget()
         self.init_menus()
 
@@ -52,10 +58,11 @@ class PyTrailerWidget(QMainWindow):
         self.refreshTimer.start(1000)
         self.setWindowTitle(self.tr("PyTrailer - Apple Trailer Downloader"))
 
+        centralWidget = QWidget()
         hbox = QHBoxLayout()
         group = QButtonGroup(hbox)
         group.setExclusive(True)
-        for cat in categories.keys():
+        for cat, url in categories:
             but = QPushButton(cat, self)
             but.setCheckable(True)
             group.addButton(but)
@@ -63,25 +70,34 @@ class PyTrailerWidget(QMainWindow):
 
         group.buttonClicked.connect(self.groupChange)
 
-        self.scrollArea = QScrollArea(self)
+        self.scrollArea = QScrollArea(centralWidget)
         scrollArea = self.scrollArea
-        self.scrolledWidget = QWidget(self)
+        self.scrolledWidget = QWidget(scrollArea)
         scrolledWidget = self.scrolledWidget
         self.hackTmp = scrolledWidget
-        scrollArea.setSizePolicy(QSizePolicy.Ignored,
-                QSizePolicy.Ignored)
 
+
+        statusView = QTreeWidget(centralWidget)
+        statusView.setMaximumHeight(200)
+        statusView.setHeaderLabels(["Trailer","Download status"])
+        statusView.setVisible(False)
+        statusView.setRootIsDecorated(False)
+        statusView.setColumnWidth(0, self.width()-200)
+        self.statusView = statusView
         vbox = QVBoxLayout()
         vbox.addLayout(hbox)
         vbox.addWidget(scrollArea)
+        vbox.addWidget(statusView)
         mlistLayout = QVBoxLayout()
         scrolledWidget.setLayout(mlistLayout)
         scrollArea.setWidget(scrolledWidget)
         scrollArea.setWidgetResizable(True)
-        scrollArea.setMinimumSize(QSize(400,150))
+        scrollArea.setMinimumSize(QSize(400,350))
+
         self.mainArea = mlistLayout
         self.loadGroup("Just added")
-        self.setCentralWidget(scrollArea)
+        centralWidget.setLayout(vbox)
+        self.setCentralWidget(centralWidget)
 
     def init_menus(self):
         fileMenu = self.menuBar().addMenu(self.tr("&File"));
@@ -95,7 +111,7 @@ class PyTrailerWidget(QMainWindow):
 
         movieMenu = self.menuBar().addMenu(self.tr("&Movies"))
         i=1
-        for cat in categories.keys():
+        for cat, url in categories:
             movieMenu.addAction(self.tr(cat), self.slotCreate(cat),
                                 QKeySequence(self.tr("F%d" % i,
                                 "Movies|%s" % cat)))
@@ -126,9 +142,12 @@ class PyTrailerWidget(QMainWindow):
             widget = self.mainArea.takeAt(0)
 
     def loadGroup(self, groupName):
+        url = None
+        for cat, catURL in categories:
+            if cat == groupName:
+                url = "http://trailers.apple.com%s" % catURL
         self.unloadCurrentGroup()
 
-        url = "http://trailers.apple.com%s" % categories[groupName]
         self.movieList = amt.getMoviesFromJSON(url)
         for i in range(len(self.movieList)):
             self.readAheadTaskQueue.put((i, self.movieList[i]))
@@ -141,6 +160,7 @@ class PyTrailerWidget(QMainWindow):
 
         for movie in self.movieList:
             w=MovieItemWidget(movie, filt, self.scrollArea)
+            w.setVisible(False)
             w.downloadClicked.connect(self.downloadTrailer)
             self.movieDict[movie.title] = w
             self.mainArea.addWidget(w)
@@ -151,11 +171,13 @@ class PyTrailerWidget(QMainWindow):
 
     def closeEvent(self, closeEvent):
         self.saveConfig()
+        self.downloader.stop()
         for p in self.readAheadProcess:
             p.terminate()
 
     def refreshMovies(self):
         changed = False
+        self.refreshDownloadStatus()
         while not self.readAheadDoneQueue.empty():
             i, updatedMovie = self.readAheadDoneQueue.get_nowait()
             oldMovie = self.movieList[i]
@@ -167,14 +189,38 @@ class PyTrailerWidget(QMainWindow):
                 w = self.movieDict[oldMovie.title]
                 if w is not None:
                     w.refresh()
+                    w.setVisible(True)
+
+    def refreshDownloadStatus(self):
+
+        for i in self.trailerDownloadDict.keys():
+            item = self.trailerDownloadDict[i]
+            trailerName = item.url.split('/')[-1]
+            statusText = None
+            if item.status == DownloadStatus.IN_PROGRESS:
+                statusText = "%d %% done" % item.percent
+            elif item.status == DownloadStatus.DONE:
+                statusText = "Done"
+            elif item.status == DownloadStatus.ERROR:
+                statusText = "Error"
+            else:
+                statusText = "Unknown"
+
+            match = self.statusView.findItems(trailerName,Qt.MatchExactly)
+            if match and len(match) > 0:
+                match = match[0]
+                match.setText(1, statusText)
+            else:
+                match = QTreeWidgetItem([trailerName, statusText])
+                self.statusView.addTopLevelItem(match)
+
+
+        if len(self.trailerDownloadDict.keys()) and not self.statusView.isVisible():
+            self.statusView.setVisible(True)
 
     def downloadTrailer(self, url):
-        subprocess.Popen(['wget','-cN',
-                          '-U',
-                          'QuickTime/7.6.2 (qtver=7.6.2;os=Windows NT 5.1Service Pack 3)',
-                          url,
-                          '-P',
-                          self.config.get("DEFAULT","downloadDir")])
+        self.trailerDownloadQueue.put((str(url),
+                                      self.config.get("DEFAULT","downloadDir")))
 
 
 class TrailerFilter(object):
@@ -211,6 +257,7 @@ def movieReadAhead(taskQueue, doneQueue):
         except:
             raise
 
+
 if __name__ == "__main__":
     app = QApplication(sys.argv)
     widget = PyTrailerWidget()
