@@ -1,6 +1,4 @@
-import subprocess
 import pickle
-import os
 import time
 import locale
 
@@ -66,8 +64,6 @@ class MovieItemWidget(QFrame):
         self.setLayout(topLevelLayout)
 
     def refresh(self):
-        if not self.isVisible():
-            return
 
         movie = self.movie
 
